@@ -26,7 +26,7 @@ __copyright__ = """
 2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 """
 
-__version__ = "0.0.3"
+__version__ = "0.0.4"
 repo_root = pathlib.Path(__file__).parent
 install_requires = (repo_root / "requirements.txt").read_text(encoding="utf-8").split('\n')
-extras_require = {"all": []}
+extras_require = {}
